/* wptKeyRevokeDlg.cpp - Key revocation dialog
 *	Copyright (C) 2001, 2002, 2003, 2005 Timo Schulz
 *	Copyright (C) 2005 g10 Code GmbH
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>

#include "resource.h"
#include "gpgme.h"
#include "wptErrors.h"
#include "wptGPG.h"
#include "wptW32API.h"
#include "wptTypes.h"
#include "wptCommonCtl.h"
#include "wptContext.h" /* for passphrase_s */
#include "wptDlgs.h"
#include "wptNLS.h"
#include "wptUTF8.h"


/* Generate a file template for the cert based on the key 
   with the keyid @keyid. */
static void
mk_cert_fname (const char *keyid, char *fname, size_t flen)
{
    gpgme_key_t k;
    size_t i;

    get_pubkey (keyid, &k);
    _snprintf (fname, flen-1, "%s-RevocationCert.asc", k->uids->name);
    for (i=0; i < strlen (fname); i++)
	if (fname[i] == ' ')
	    fname[i] = '_';
}


/* Release the cert data and store it in the file @fname. */
static void
release_cert_as_file (char *revcert, const char *fname)
{
    gpgme_error_t err;
    gpgme_data_t rev;

    err = gpgme_data_new_from_mem (&rev, revcert, strlen (revcert), 1);
    if (!err)
	gpg_data_release_and_set_file (rev, fname);
    else
	msg_box (NULL, gpgme_strerror (err), _("Key Revocation"), MB_ERR);
    free (revcert);
}


/* Dialog box procedure for key revocation. */
BOOL CALLBACK
key_revoke_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{	
    static winpt_key_t k;
    gpgme_error_t err;
    HWND list;	
    int idx, use_desc;
    char desc[256], file[256], *p = NULL;
    char pwd[256];
    char *inp_data = NULL, *revcert=NULL;
    
    switch( msg ) {
    case WM_INITDIALOG:
        if( !lparam )
            dlg_fatal_error(dlg, "Could not get dialog param!");
        k = (winpt_key_t )lparam;
        SetWindowText (dlg, _("Key Revocation"));
	SetDlgItemText (dlg, IDCANCEL, _("&Cancel"));
	SetDlgItemText (dlg, IDC_KEYREVOKE_REASONINF, _("Reason for revocation"));
	SetDlgItemText (dlg, IDC_KEYREVOKE_OPTINF, _("Optional description text"));
	SetDlgItemText (dlg, IDC_KEYREVOKE_PWDINF, _("&Passphrase"));
	SetDlgItemText (dlg, IDC_KEYREVOKE_OUTINF, _("Output file"));
        SetDlgItemText (dlg, IDC_KEYREVOKE_HINT,
                        _("Please move this certificate to a medium where it can be "
                          "stored in a safe place (floppy, CDR, etc..). "
                          "If an attacker gets access to this certificate he can use it to "
                          "render your key unusable!"));
        list = GetDlgItem( dlg, IDC_KEYREVOKE_REASON );
	listbox_add_string (list, _("0. No reason specified"));
        listbox_add_string (list, _("1. Key has been compromised"));
        listbox_add_string (list, _("2. Key is superseded"));
        listbox_add_string (list, _("3. Key is no longer used"));
        SetForegroundWindow (dlg);
	center_window (dlg, NULL);
        return TRUE;
        
    case WM_SYSCOMMAND:
        if( LOWORD( wparam ) == SC_CLOSE ) {
            SetDlgItemText( dlg, IDC_KEYREVOKE_PWD, "" );
            EndDialog( dlg, TRUE );
        }
        return FALSE;
        
    case WM_COMMAND:
        switch( LOWORD( wparam ) ) {
	case IDC_KEYREVOKE_CHOOSE:
	    const char *s;
	    mk_cert_fname (k->keyid, file, sizeof file-1);
	    s = get_filesave_dlg (dlg, _("Choose File to save the Certificate"), NULL, file);
	    if (s && *s)
		SetDlgItemText (dlg, IDC_KEYREVOKE_FILE, s);
	    return TRUE;

        case IDOK:
            list = GetDlgItem (dlg, IDC_KEYREVOKE_REASON);
            idx = SendMessage (list, LB_GETCURSEL, 0, 0);
            if (idx < 0 || idx > 3) {
		msg_box (dlg, _("Please select a reason."), 
			 _("Key Revocation"), MB_ERR);
                return TRUE;
            }
            if (!GetDlgItemText (dlg, IDC_KEYREVOKE_FILE, file, sizeof (file)-1)) {
		msg_box (dlg, _("Please enter a file name."), 
			 _("Key Revocation"), MB_ERR);
                return TRUE;
            }
            use_desc = 1;
            if (!GetDlgItemText (dlg, IDC_KEYREVOKE_TEXT, desc, sizeof (desc)-1))
                use_desc = 0;
            if( !GetDlgItemText (dlg, IDC_KEYREVOKE_PWD, pwd, sizeof (pwd)-1)) {
		msg_box (dlg, _("Please enter the passphrase."), _("Key Revocation"), MB_ERR);
                return TRUE;
            }
          
	    if (use_desc)
		p = wincp_to_utf8 (desc, strlen (desc));
	    /* we use the keyid to avoid charset problems and UTF8 encodings.*/
	    inp_data = generate_revoc_input (idx, use_desc? p : NULL, pwd);
            
            err = gpg_revoke_key (inp_data, k->keyid, &revcert);
	    wipememory (&pwd, sizeof (pwd));
	    memset (inp_data, 0, strlen (inp_data));
	    free (inp_data);
	    free_if_alloc (p);

            if (err) {
                msg_box (dlg, gpgme_strerror (err), _("Key Revocation"), MB_ERR);
		if (revcert)
		    free (revcert);
	    }
	    else {
		msg_box (dlg, _("Revocation certificate generated."), _("GnuPG Status"), MB_OK);
		release_cert_as_file (revcert, file);
	    }
	    EndDialog (dlg, TRUE);
            return TRUE;
            
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return FALSE;
        }
        break;
    }
    
    return FALSE;
}
